# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ...ui import IEditor

from .layout_adapter import LayoutAdapter


class EditorLayoutAdapter(LayoutAdapter):
    """ The EditorLayoutAdapter class is a base class for adapters than
    implement the :class:`~dip.pui.IEditor` interface for Qt layouts.
    """

    def remove_label(self):
        """ Remove an editor's label.  This will only be called if the label
        policy is 'optional'.  This default implementation does nothing.
        """

    @IEditor.read_only.getter
    def read_only(self):
        """ Invoked to get the read-only state. """

        # It is read-only if every top-level QWidget within the layout is
        # read-only.
        for w in self.qt4_widgets(self.adaptee):
            if hasattr(w, 'isReadOnly'):
                if not w.isReadOnly():
                    return False
            elif w.isEnabled():
                return False

        return True

    @read_only.setter
    def read_only(self, value):
        """ Invoked to set the read-only state. """

        for w in self.qt4_widgets(self.adaptee):
            if hasattr(w, 'setReadOnly'):
                w.setReadOnly(value)
            else:
                w.setEnabled(not value)
