# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ...ui import ISingleViewContainer

from .widget_adapter import WidgetAdapter


class SingleViewContainerAdapter(WidgetAdapter):
    """ The SingleViewContainerAdapter class is a base class for adapters
    than implement the :class:`~dip.pui.ISingleViewContainer` interface.
    """

    @ISingleViewContainer.views.getter
    def views(self):
        """ Invoked to get the views. """

        view = self.view

        return () if view is None else (view, )

    @views.setter
    def views(self, value):
        """ Invoked to set the views. """

        if len(value) > 1:
            raise ValueError(
                    "{0} can only contain a single view".format(
                            type(self.adaptee).__name__))

        self.view = value[0] if len(value) != 0 else None
