# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QLayout, QWidget


def find_visual_item(root, types, name=None, recurse=True):
    """ Find an item in a visual hierachy.  the item is determined by its type
    and, optionally, its name.
    """

    for itm in visual_items(root, recurse):
        if isinstance(itm, types):
            if name is None or itm.objectName() == name:
                return itm

    return None


def visual_items(root, recurse=True):
    """ A generator for all layouts and widgets starting at the root of a
    visual hierachy.  The order is breadth first.
    """

    if isinstance(root, QWidget):
        yield root
        layout = root.layout()
    elif isinstance(root, QLayout):
        layout = root
    else:
        layout = None

    if layout is not None:
        yield layout

        for child in _visual_layout_items(layout, recurse):
            yield child


def _visual_layout_items(layout, recurse):
    """ A generator for all layouts and widgets contained in a layout.  The
    order is breadth first.
    """

    children = []

    for idx in range(layout.count()):
        itm = layout.itemAt(idx)

        w = itm.widget()
        if w is not None:
            children.append(w)
        else:
            l = itm.layout()
            if l is not None:
                children.append(l)

    # Breadth first.
    for child in children:
        yield child

    if recurse:
        # Now descend into each child.
        for child in children:
            if isinstance(child, QWidget):
                sublayout = child.layout()
                if sublayout is None:
                    continue

            else:
                sublayout = child

            for grandchild in _visual_layout_items(sublayout, True):
                yield grandchild
