# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Instance, List, Str

from .i_action import IAction
from .container_factory import ContainerFactory
from .i_controller import IController
from .i_menu import IMenu
from .i_view_binding import IViewBinding


class Menu(ContainerFactory):
    """ The Menu class implements a factory for menus that implement or can be
    adapted to :class:`~dip.ui.IMenu`.
    """

    # The contents of the view.
    contents = List(Instance('dip.ui.Menu', Str()))

    # The menu's title.
    title = Str()

    # The specific interface implemented by the view.
    view_interface = IMenu

    # Menu's are hidden by default.  They will be visible when they contain
    # something that is visible.
    visible = False

    def create_subviews(self, model, controller, view):
        """ Create the sub-views for a containing view.

        :param model:
            is the model.
        :param controller:
            is the controller.
        :param view:
            is the containing view.
        :return:
            the sub-views.
        """

        items = []

        for content in self.contents:
            if isinstance(content, Menu):
                item = content.create(model, controller, view)
                content.finalize(item)
            elif content != '':
                # Look for an action.
                item = controller.find_action(content)
                if item is None:
                    item = content
            else:
                # An empty string means a separator.
                item = ''

            items.append(item)

        return items

    def configure(self, view):
        """ Reimplemented to configure the IMenu specific attributes.

        :param view:
            is the view to configure.
        """

        IMenu(view).title = self.title

        super(Menu, self).configure(view)
